@@ -288,7 +288,7 @@ class SeqLinear(nn.Module):
         self.n_heads = n_heads
         self.device = device
         
-        d_in_proj = 2 * self.d_state + self.d_inner + 2 * self.n_heads
+        d_in_proj = 2 * self.d_state + self.d_inner + self.n_heads
         self.in_proj = nn.Linear(d_model, d_in_proj, bias=False, device=device)
         conv_dim = 2 * self.d_state + self.d_inner
         self.conv = nn.Conv1d(
@@ -316,20 +316,33 @@ class SeqLinear(nn.Module):
     def bidirectional(self, x):
         bsz, src_len, d_model = x.size()
         
-        xIOwg = self.in_proj(x)
+        xIOw = self.in_proj(x)
         
-        xIO, w, g = torch.split(
-            xIOwg,
+        xIO, w = torch.split(
+            xIOw,
             [
                 2 * self.d_state + self.d_inner,
                 self.n_heads,
-                self.n_heads,
             ],
             dim=-1
         )
-        
-        xIO_f = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
-        xIO_b = self.conv(xIO.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+
+        if cuda_causal_conv1d is not None:
+            xIO_f = cuda_causal_conv1d(
+                xIO.transpose(1, 2).contiguous(),
+                self.conv.weight.squeeze(1),
+                bias=self.conv.bias,
+                activation=None,
+            ).transpose(1, 2)
+            xIO_b = cuda_causal_conv1d(
+                xIO.flip(1).transpose(1, 2).contiguous(),
+                self.conv.weight.squeeze(1),
+                bias=self.conv.bias,
+                activation=None,
+            ).transpose(1, 2)
+        else:
+            xIO_f = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+            xIO_b = self.conv(xIO.flip(1).transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
         
         x_f, I_f, O_f = torch.split(
             xIO_f,
@@ -358,7 +371,6 @@ class SeqLinear(nn.Module):
         I_b = rearrange(I_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         O_b = rearrange(O_b, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
 
-        w = F.softplus(w)
         decay_f = F.silu(w * self.w_base).transpose(1, 2).unsqueeze(-2).repeat(1, 1, src_len, 1)
         decay_b = decay_f.flip(1)
         mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=-1)
@@ -374,8 +386,7 @@ class SeqLinear(nn.Module):
         out_f = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x_f, decay_f, I_f, O_f)
         out_b = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x_b, decay_b, I_b, O_b)
         
-        out = (out_f + out_b) * F.silu(g).unsqueeze(-1)
-        out = rearrange(out, 'b s h d -> b s (h d)')
+        out = rearrange(out_f + out_b, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
     
@@ -383,19 +394,26 @@ class SeqLinear(nn.Module):
         if mode == "bidirectional": return self.bidirectional(x)
         bsz, src_len, d_model = x.size()
 
-        xIOwg = self.in_proj(x)
+        xIOw = self.in_proj(x)
         
-        xIO, w, g = torch.split(
-            xIOwg,
+        xIO, w = torch.split(
+            xIOw,
             [
                 2 * self.d_state + self.d_inner,
                 self.n_heads,
-                self.n_heads,
             ],
             dim=-1
         )
         
-        xIO = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
+        if cuda_causal_conv1d is not None:
+            xIO = cuda_causal_conv1d(
+                xIO.transpose(1, 2).contiguous(),
+                self.conv.weight.squeeze(1),
+                bias=self.conv.bias,
+                activation=None,
+            ).transpose(1, 2)
+        else:
+            xIO = self.conv(xIO.transpose(1, 2).contiguous()).transpose(1, 2)[:, :src_len]
         
         x, I, O = torch.split(
             xIO,
@@ -407,7 +425,6 @@ class SeqLinear(nn.Module):
             dim=-1
         )
         
-        w = F.softplus(w)
         x = rearrange(x, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         I = rearrange(I, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
         O = rearrange(O, 'b s (h d) -> b s h d', h=self.n_heads).contiguous()
@@ -420,16 +437,11 @@ class SeqLinear(nn.Module):
             mask = torch.tril(torch.ones(src_len, src_len, dtype=torch.bool, device=self.device), diagonal=0)
             decay = decay.masked_fill(~mask, float('-inf'))
             decay = F.softmax(decay, dim=-1)
-            import matplotlib.pyplot as plt
-            plt.clf()
-            plt.plot(decay[0, :, -1].transpose(0, 1).detach().cpu().numpy())
-            plt.pause(0.01)
             out = torch.einsum('bthd, bhts, bshd, bshe -> bthe', x, decay, I, O)
         elif mode == "noncausal":
             decay = F.softmax(-F.silu(w * self.w_base), dim=1)
             out = torch.einsum('bthd, bsh, bshd, bshe -> bthe', x, decay, I, O)
 
-        out = out * F.silu(g).unsqueeze(-1)
         out = rearrange(out, 'b s h d -> b s (h d)')
         out = self.out_proj(out)
         return out
